@@ -1,19 +1,20 @@
 /*
- *  Copyright (c) Stephan Arts 2011 <stephan@xfce.org>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  Copyright (c) Stephan Arts 2006-2012 <stephan@xfce.org>
  *
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License
+ *  as published by the Free Software Foundation; either version 2
+ *  of the License, or (at your option) any later version.
+ * 
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
- *
+ *  GNU General Public License for more details.
+ * 
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 #include <config.h>
@@ -41,6 +42,15 @@ static guint rstto_thumbnail_size[] =
     THUMBNAIL_SIZE_VERY_LARGE_SIZE
 };
 
+enum
+{
+    RSTTO_FILE_SIGNAL_CHANGED = 0,
+    RSTTO_FILE_SIGNAL_COUNT
+};
+
+static gint
+rstto_file_signals[RSTTO_FILE_SIGNAL_COUNT];
+
 static void
 rstto_file_init (GObject *);
 static void
@@ -112,6 +122,7 @@ struct _RsttoFilePriv
 
     gchar *uri;
     gchar *path;
+    gchar *collate_key;
 
     gchar *thumbnail_path;
     GdkPixbuf *thumbnails[THUMBNAIL_SIZE_COUNT];
@@ -124,9 +135,9 @@ struct _RsttoFilePriv
 static void
 rstto_file_init (GObject *object)
 {
-    RsttoFile *file = RSTTO_FILE (object);
+    RsttoFile *r_file = RSTTO_FILE (object);
 
-    file->priv = g_new0 (RsttoFilePriv, 1);
+    r_file->priv = g_new0 (RsttoFilePriv, 1);
 }
 
 
@@ -142,6 +153,17 @@ rstto_file_class_init (GObjectClass *object_class)
 
     object_class->set_property = rstto_file_set_property;
     object_class->get_property = rstto_file_get_property;
+
+    rstto_file_signals[RSTTO_FILE_SIGNAL_CHANGED] = g_signal_new("changed",
+            G_TYPE_FROM_CLASS(object_class),
+            G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+            0,
+            NULL,
+            NULL,
+            g_cclosure_marshal_VOID__VOID,
+            G_TYPE_NONE,
+            0,
+            NULL);
 }
 
 /**
@@ -152,55 +174,60 @@ rstto_file_class_init (GObjectClass *object_class)
 static void
 rstto_file_dispose (GObject *object)
 {
-    RsttoFile *file = RSTTO_FILE (object);
+    RsttoFile *r_file = RSTTO_FILE (object);
     gint i = 0;
 
-    if (file->priv)
+    if (r_file->priv)
     {
-        if (file->priv->file)
+        if (r_file->priv->file)
         {
-            g_object_unref (file->priv->file);
-            file->priv->file = NULL;
+            g_object_unref (r_file->priv->file);
+            r_file->priv->file = NULL;
         }
-        if (file->priv->display_name)
+        if (r_file->priv->display_name)
         {
-            g_free (file->priv->display_name);
-            file->priv->display_name = NULL;
+            g_free (r_file->priv->display_name);
+            r_file->priv->display_name = NULL;
         }
-        if (file->priv->content_type)
+        if (r_file->priv->content_type)
         {
-            g_free (file->priv->content_type);
-            file->priv->content_type = NULL;
+            g_free (r_file->priv->content_type);
+            r_file->priv->content_type = NULL;
         }
-        if (file->priv->path)
+        if (r_file->priv->path)
         {
-            g_free (file->priv->path);
-            file->priv->path = NULL;
+            g_free (r_file->priv->path);
+            r_file->priv->path = NULL;
         }
-        if (file->priv->thumbnail_path)
+        if (r_file->priv->thumbnail_path)
         {
-            g_free (file->priv->thumbnail_path);
-            file->priv->thumbnail_path = NULL;
+            g_free (r_file->priv->thumbnail_path);
+            r_file->priv->thumbnail_path = NULL;
         }
-        if (file->priv->uri)
+        if (r_file->priv->uri)
         {
-            g_free (file->priv->uri);
-            file->priv->uri = NULL;
+            g_free (r_file->priv->uri);
+            r_file->priv->uri = NULL;
+        }
+        if (r_file->priv->collate_key)
+        {
+            g_free (r_file->priv->collate_key);
+            r_file->priv->collate_key = NULL;
         }
 
         for (i = 0; i < THUMBNAIL_SIZE_COUNT; ++i)
         {
-            if (file->priv->thumbnails[i])
+            if (r_file->priv->thumbnails[i])
             {
-                g_object_unref (file->priv->thumbnails[i]);
-                file->priv->thumbnails[i] = NULL;
+                g_object_unref (r_file->priv->thumbnails[i]);
+                r_file->priv->thumbnails[i] = NULL;
             }
         }
 
-        g_free (file->priv);
-        file->priv = NULL;
+        g_free (r_file->priv);
+        r_file->priv = NULL;
 
-        open_files = g_list_remove_all (open_files, file);
+        open_files = g_list_remove_all (open_files, r_file);
     }
 }
 
@@ -226,16 +253,17 @@ rstto_file_finalize (GObject *object)
 RsttoFile *
 rstto_file_new ( GFile *file )
 {
-    RsttoFile *o_file = NULL;
+    RsttoFile *r_file = NULL;
     GList *iter = open_files;
+
     while ( NULL != iter )
     {
         /* Check if the file is already opened, if so
          * return that one.
          */
-        o_file = RSTTO_FILE (iter->data);
+        r_file = RSTTO_FILE (iter->data);
         if ( TRUE == g_file_equal (
-                o_file->priv->file,
+                r_file->priv->file,
                 file) )
         {
             g_object_ref (G_OBJECT (iter->data));
@@ -244,13 +272,13 @@ rstto_file_new ( GFile *file )
         iter = g_list_next (iter);
     }
 
-    o_file = g_object_new (RSTTO_TYPE_FILE, NULL);
-    o_file->priv->file = file;
+    r_file = g_object_new (RSTTO_TYPE_FILE, NULL);
+    r_file->priv->file = file;
     g_object_ref (file);
 
-    open_files = g_list_append (open_files, o_file);
+    open_files = g_list_append (open_files, r_file);
 
-    return o_file;
+    return r_file;
 }
 
 
@@ -273,27 +301,27 @@ rstto_file_get_property (
 }
 
 GFile *
-rstto_file_get_file ( RsttoFile *file )
+rstto_file_get_file ( RsttoFile *r_file )
 {
-    return file->priv->file;
+    return r_file->priv->file;
 }
 
 gboolean
-rstto_file_equal ( RsttoFile *a, RsttoFile *b )
+rstto_file_equal ( RsttoFile *r_file_a, RsttoFile *r_file_b )
 {
-    return a == b;
+    return r_file_a == r_file_b;
 }
 
 const gchar *
-rstto_file_get_display_name ( RsttoFile *file )
+rstto_file_get_display_name ( RsttoFile *r_file )
 {
     GFileInfo *file_info = NULL;
     const gchar *display_name;
 
-    if ( NULL == file->priv->display_name )
+    if ( NULL == r_file->priv->display_name )
     {
         file_info = g_file_query_info (
-                file->priv->file,
+                r_file->priv->file,
                 G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
                 0,
                 NULL,
@@ -303,48 +331,74 @@ rstto_file_get_display_name ( RsttoFile *file )
             display_name = g_file_info_get_display_name (file_info);
             if ( NULL != display_name )
             {
-                file->priv->display_name = g_strdup (display_name);
+                r_file->priv->display_name = g_strdup (display_name);
             }
             g_object_unref (file_info);
         }
     }
 
-    return (const gchar *)file->priv->display_name;
+    return (const gchar *)r_file->priv->display_name;
 }
 
 const gchar *
-rstto_file_get_path ( RsttoFile *file )
+rstto_file_get_path ( RsttoFile *r_file )
 {
-    g_return_val_if_fail (RSTTO_IS_FILE (file), NULL);
-    g_return_val_if_fail (G_IS_FILE (file->priv->file), NULL);
+    g_return_val_if_fail (RSTTO_IS_FILE (r_file), NULL);
+    g_return_val_if_fail (G_IS_FILE (r_file->priv->file), NULL);
 
-    if ( NULL == file->priv->path )
+    if ( NULL == r_file->priv->path )
     {
-        file->priv->path = g_file_get_path (file->priv->file);
+        r_file->priv->path = g_file_get_path (r_file->priv->file);
     }
-    return (const gchar *)file->priv->path;
+    return (const gchar *)r_file->priv->path;
 }
 
 const gchar *
-rstto_file_get_uri ( RsttoFile *file )
+rstto_file_get_uri ( RsttoFile *r_file )
 {
-    if ( NULL == file->priv->uri )
+    if ( NULL == r_file->priv->uri )
     {
-        file->priv->uri = g_file_get_uri (file->priv->file);
+        r_file->priv->uri = g_file_get_uri (r_file->priv->file);
     }
-    return (const gchar *)file->priv->uri;
+    return (const gchar *)r_file->priv->uri;
 }
 
 const gchar *
-rstto_file_get_content_type ( RsttoFile *file )
+rstto_file_get_collate_key ( RsttoFile *r_file )
+{
+    if ( NULL == r_file->priv->collate_key )
+    {
+        gchar *basename = g_file_get_basename (rstto_file_get_file (r_file));
+        if ( NULL != basename )
+        {
+            /* If we can use casefold for case insenstivie sorting, then
+             * do so */
+            gchar *casefold = g_utf8_casefold (basename, -1);
+            if ( NULL != casefold )
+            {
+                r_file->priv->collate_key = g_utf8_collate_key_for_filename (casefold, -1);
+                g_free (casefold);
+            }
+            else
+            {
+                r_file->priv->collate_key = g_utf8_collate_key_for_filename (basename, -1);
+            }
+            g_free (basename);
+        }
+    }
+    return (const gchar *)r_file->priv->collate_key;
+}
+
+const gchar *
+rstto_file_get_content_type ( RsttoFile *r_file )
 {
     GFileInfo *file_info = NULL;
     const gchar *content_type;
 
-    if ( NULL == file->priv->content_type )
+    if ( NULL == r_file->priv->content_type )
     {
         file_info = g_file_query_info (
-                file->priv->file,
+                r_file->priv->file,
                 "standard::content-type",
                 0,
                 NULL,
@@ -354,20 +408,20 @@ rstto_file_get_content_type ( RsttoFile *file )
             content_type = g_file_info_get_content_type (file_info);
             if ( NULL != content_type )
             {
-                file->priv->content_type = g_strdup (content_type);
+                r_file->priv->content_type = g_strdup (content_type);
             }
             g_object_unref (file_info);
         }
     }
 
-    return (const gchar *)file->priv->content_type;
+    return (const gchar *)r_file->priv->content_type;
 }
 
 guint64
-rstto_file_get_modified_time ( RsttoFile *file )
+rstto_file_get_modified_time ( RsttoFile *r_file )
 {
     guint64 time_ = 0;
-    GFileInfo *file_info = g_file_query_info (file->priv->file, "time::modified", 0, NULL, NULL);
+    GFileInfo *file_info = g_file_query_info (r_file->priv->file, "time::modified", 0, NULL, NULL);
 
     time_ = g_file_info_get_attribute_uint64 ( file_info, "time::modified" );
 
@@ -377,63 +431,70 @@ rstto_file_get_modified_time ( RsttoFile *file )
 }
 
 ExifEntry *
-rstto_file_get_exif ( RsttoFile *file, ExifTag id )
+rstto_file_get_exif ( RsttoFile *r_file, ExifTag id )
 {
     /* If there is no exif-data object, try to create it */
-    if ( NULL == file->priv->exif_data )
+    if ( NULL == r_file->priv->exif_data )
     {
-        file->priv->exif_data = exif_data_new_from_file ( rstto_file_get_path (file) );
+        r_file->priv->exif_data = exif_data_new_from_file (
+                rstto_file_get_path (r_file) );
     }
-    if ( NULL != file->priv->exif_data )
+
+    if ( NULL != r_file->priv->exif_data )
     {
         return exif_data_get_entry (
-                file->priv->exif_data,
+                r_file->priv->exif_data,
                 id );
     }
+
+    /* If there is no exif-data, return NULL */
     return NULL;
 }
 
 RsttoImageOrientation
-rstto_file_get_orientation ( RsttoFile *file )
+rstto_file_get_orientation ( RsttoFile *r_file )
 {
     ExifEntry *exif_entry = NULL;
-    if (file->priv->orientation == 0 )
+    if (r_file->priv->orientation == 0 )
     {
-        exif_entry = rstto_file_get_exif (file, EXIF_TAG_ORIENTATION);
+        /* Try to get the default orientation from the EXIF tag */
+        exif_entry = rstto_file_get_exif (r_file, EXIF_TAG_ORIENTATION);
         if (NULL != exif_entry)
         {
-            file->priv->orientation = exif_get_short (
+            r_file->priv->orientation = exif_get_short (
                     exif_entry->data,
                     exif_data_get_byte_order (exif_entry->parent->parent));
 
             exif_entry_free (exif_entry);
         }
-        if (file->priv->orientation == 0)
+
+        /* If the orientation-tag is not set, default to NONE */
+        if (r_file->priv->orientation == 0)
         {
             /* Default orientation */
-            file->priv->orientation = RSTTO_IMAGE_ORIENT_NONE;
+            r_file->priv->orientation = RSTTO_IMAGE_ORIENT_NONE;
         }
 
     }
-    return file->priv->orientation;
+    return r_file->priv->orientation;
 }
 
 void
 rstto_file_set_orientation (
-        RsttoFile *file ,
+        RsttoFile *r_file ,
         RsttoImageOrientation orientation )
 {
-    file->priv->orientation = orientation;
+    r_file->priv->orientation = orientation;
 }
 
 gboolean
-rstto_file_has_exif ( RsttoFile *file )
+rstto_file_has_exif ( RsttoFile *r_file )
 {
-    if ( NULL == file->priv->exif_data )
+    if ( NULL == r_file->priv->exif_data )
     {
-        file->priv->exif_data = exif_data_new_from_file ( rstto_file_get_path (file) );
+        r_file->priv->exif_data = exif_data_new_from_file ( rstto_file_get_path (r_file) );
     }
-    if ( NULL == file->priv->exif_data )
+    if ( NULL == r_file->priv->exif_data )
     {
         return FALSE;
     }
@@ -441,40 +502,63 @@ rstto_file_has_exif ( RsttoFile *file )
 }
 
 const gchar *
-rstto_file_get_thumbnail_path ( RsttoFile *file)
+rstto_file_get_thumbnail_path ( RsttoFile *r_file)
 {
     const gchar *uri;
     gchar *checksum;
     gchar *filename;
 
-    if (NULL == file->priv->thumbnail_path)
+    if (NULL == r_file->priv->thumbnail_path)
     {
-        uri = rstto_file_get_uri (file);
+        uri = rstto_file_get_uri (r_file);
         checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, strlen (uri));
         filename = g_strconcat (checksum, ".png", NULL);
 
-        file->priv->thumbnail_path = g_build_path ("/", g_get_home_dir(), ".thumbnails", "normal", filename, NULL);
+        /* build and check if the thumbnail is in the new location */
+        r_file->priv->thumbnail_path = g_build_path ("/", g_get_user_cache_dir(), "thumbnails", "normal", filename, NULL);
+
+        if(!g_file_test (r_file->priv->thumbnail_path, G_FILE_TEST_EXISTS))
+        {
+            /* Fallback to old version */
+            g_free (r_file->priv->thumbnail_path);
+
+            r_file->priv->thumbnail_path = g_build_path ("/", g_get_home_dir(), ".thumbnails", "normal", filename, NULL);
+            if(!g_file_test (r_file->priv->thumbnail_path, G_FILE_TEST_EXISTS))
+            {
+                /* Thumbnail doesn't exist in either spot */
+                g_free (r_file->priv->thumbnail_path);
+                r_file->priv->thumbnail_path = NULL;
+            }
+        }
 
         g_free (checksum);
         g_free (filename);
     }
 
-    return file->priv->thumbnail_path;
+    return r_file->priv->thumbnail_path;
 }
 
 const GdkPixbuf *
-rstto_file_get_thumbnail ( RsttoFile *file , RsttoThumbnailSize size)
+rstto_file_get_thumbnail (
+        RsttoFile *r_file,
+        RsttoThumbnailSize size )
 {
     const gchar *thumbnail_path;
     RsttoThumbnailer *thumbnailer;
 
-    if (file->priv->thumbnails[size])
-        return file->priv->thumbnails[size];
+    if (r_file->priv->thumbnails[size])
+        return r_file->priv->thumbnails[size];
 
-    thumbnail_path = rstto_file_get_thumbnail_path (file);
+    thumbnail_path = rstto_file_get_thumbnail_path (r_file);
 
     thumbnailer = rstto_thumbnailer_new();
-    rstto_thumbnailer_queue_file (thumbnailer, file);
+    rstto_thumbnailer_queue_file (thumbnailer, r_file);
+
+    if (NULL == thumbnail_path)
+    {
+        /* No thumbnail to return at this time */
+        return NULL;
+    }
 
     /* FIXME:
      *
@@ -482,7 +566,7 @@ rstto_file_get_thumbnail ( RsttoFile *file , RsttoThumbnailSize size)
      * of the thumbnailer, to account for changed thumbnails
      * aswell as missing ones.
      */
-    file->priv->thumbnails[size] = gdk_pixbuf_new_from_file_at_scale (
+    r_file->priv->thumbnails[size] = gdk_pixbuf_new_from_file_at_scale (
             thumbnail_path,
             rstto_thumbnail_size[size],
             rstto_thumbnail_size[size],
@@ -491,5 +575,15 @@ rstto_file_get_thumbnail ( RsttoFile *file , RsttoThumbnailSize size)
 
     g_object_unref (thumbnailer);
 
-    return file->priv->thumbnails[size];
+    return r_file->priv->thumbnails[size];
+}
+
+void
+rstto_file_changed ( RsttoFile *r_file )
+{
+    g_signal_emit (
+            G_OBJECT (r_file),
+            rstto_file_signals[RSTTO_FILE_SIGNAL_CHANGED],
+            0,
+            NULL);
 }
