@@ -1382,7 +1382,7 @@ rstto_icon_bar_calculate_item_size (
     gint       focus_pad;
     gint       int_pad;
 
-    if (G_LIKELY (item->width != -1 && item->width != -1))
+    if (G_LIKELY (item->width != -1))
         return;
 
     gtk_widget_style_get (GTK_WIDGET (icon_bar),
