@@ -1,19 +1,20 @@
 /*
- *  Copyright (c) Stephan Arts 2009-2011 <stephan@xfce.org>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  Copyright (c) Stephan Arts 2006-2012 <stephan@xfce.org>
  *
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License
+ *  as published by the Free Software Foundation; either version 2
+ *  of the License, or (at your option) any later version.
+ * 
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
- *
+ *  GNU General Public License for more details.
+ * 
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 #include <config.h>
@@ -60,7 +61,8 @@ enum
     PROP_SHOW_THUMBNAILBAR,
     PROP_SHOW_STATUSBAR,
     PROP_SHOW_CLOCK,
-    PROP_HIDE_THUMBNAILBAR_FULLSCREEN,
+    PROP_LIMIT_QUALITY,
+    PROP_HIDE_THUMBNAILS_FULLSCREEN,
     PROP_WINDOW_WIDTH,
     PROP_WINDOW_HEIGHT,
     PROP_BGCOLOR,
@@ -70,7 +72,7 @@ enum
     PROP_SLIDESHOW_TIMEOUT,
     PROP_WRAP_IMAGES,
     PROP_DESKTOP_TYPE,
-    PROP_REVERT_ZOOM_DIRECTION,
+    PROP_INVERT_ZOOM_DIRECTION,
     PROP_USE_THUNAR_PROPERTIES,
     PROP_MAXIMIZE_ON_STARTUP,
     PROP_ERROR_MISSING_THUMBNAILER,
@@ -112,9 +114,10 @@ struct _RsttoSettingsPriv
     gboolean  show_thumbnailbar;
     gboolean  show_statusbar;
     gboolean  show_clock;
-    gboolean  hide_thumbnailbar_fullscreen;
+    gboolean  limit_quality;
+    gboolean  hide_thumbnails_fullscreen;
     gchar    *navigationbar_position;
-    gboolean  revert_zoom_direction;
+    gboolean  invert_zoom_direction;
     guint     window_width;
     guint     window_height;
     gchar    *last_file_path;
@@ -166,6 +169,8 @@ rstto_settings_init (GObject *object)
 
         gtk_accel_map_change_entry ("<Window>/delete", GDK_Delete, GDK_SHIFT_MASK, FALSE);
 
+        gtk_accel_map_change_entry ("<Window>/refresh", GDK_r, GDK_CONTROL_MASK, FALSE);
+
         gtk_accel_map_change_entry ("<Actions>/RsttoWindow/play", GDK_F5, 0, FALSE);
     }
     
@@ -181,7 +186,7 @@ rstto_settings_init (GObject *object)
     settings->priv->show_statusbar = TRUE;
     settings->priv->use_thunar_properties = TRUE;
     settings->priv->maximize_on_startup = TRUE;
-    settings->priv->hide_thumbnailbar_fullscreen = TRUE;
+    settings->priv->hide_thumbnails_fullscreen = TRUE;
     settings->priv->errors.missing_thumbnailer = TRUE;
     settings->priv->thumbnail_size = THUMBNAIL_SIZE_NORMAL;
 
@@ -252,7 +257,7 @@ rstto_settings_init (GObject *object)
             "/window/thumbnails/hide-fullscreen",
             G_TYPE_BOOLEAN,
             settings,
-            "hide-thumbnailbar-fullscreen");
+            "hide-thumbnails-fullscreen");
 
     xfconf_g_property_bind (
             settings->priv->channel,
@@ -281,10 +286,10 @@ rstto_settings_init (GObject *object)
 
     xfconf_g_property_bind (
             settings->priv->channel,
-            "/window/revert-zoom-direction",
+            "/window/invert-zoom-direction",
             G_TYPE_BOOLEAN,
             settings,
-            "revert-zoom-direction");
+            "invert-zoom-direction");
 
     xfconf_g_property_bind (
             settings->priv->channel,
@@ -300,6 +305,13 @@ rstto_settings_init (GObject *object)
             settings,
             "wrap-images");
 
+    xfconf_g_property_bind (
+            settings->priv->channel,
+            "/image/limit-quality",
+            G_TYPE_BOOLEAN,
+            settings,
+            "limit-quality");
+
     xfconf_g_property_bind (
             settings->priv->channel,
             "/window/use-thunar-properties",
@@ -417,14 +429,25 @@ rstto_settings_class_init (GObjectClass *object_class)
             pspec);
 
     pspec = g_param_spec_boolean (
-            "hide-thumbnailbar-fullscreen",
+            "limit-quality",
             "",
             "",
             TRUE,
             G_PARAM_READWRITE);
     g_object_class_install_property (
             object_class,
-            PROP_HIDE_THUMBNAILBAR_FULLSCREEN,
+            PROP_LIMIT_QUALITY,
+            pspec);
+
+    pspec = g_param_spec_boolean (
+            "hide-thumbnails-fullscreen",
+            "",
+            "",
+            TRUE,
+            G_PARAM_READWRITE);
+    g_object_class_install_property (
+            object_class,
+            PROP_HIDE_THUMBNAILS_FULLSCREEN,
             pspec);
 
     pspec = g_param_spec_string (
@@ -439,14 +462,14 @@ rstto_settings_class_init (GObjectClass *object_class)
             pspec);
 
     pspec = g_param_spec_boolean (
-            "revert-zoom-direction",
+            "invert-zoom-direction",
             "",
             "",
             TRUE,
             G_PARAM_READWRITE);
     g_object_class_install_property (
             object_class,
-            PROP_REVERT_ZOOM_DIRECTION,
+            PROP_INVERT_ZOOM_DIRECTION,
             pspec);
 
     pspec = g_param_spec_string (
@@ -680,24 +703,27 @@ rstto_settings_set_property    (GObject      *object,
         case PROP_SHOW_CLOCK:
             settings->priv->show_clock = g_value_get_boolean (value);
             break;
-        case PROP_HIDE_THUMBNAILBAR_FULLSCREEN:
-            settings->priv->hide_thumbnailbar_fullscreen = g_value_get_boolean (value);
+        case PROP_LIMIT_QUALITY:
+            settings->priv->limit_quality = g_value_get_boolean (value);
+            break;
+        case PROP_HIDE_THUMBNAILS_FULLSCREEN:
+            settings->priv->hide_thumbnails_fullscreen = g_value_get_boolean (value);
             break;
         case PROP_NAVBAR_POSITION:
             str_val = g_value_get_string (value);
 
-            if ((!g_strcasecmp (str_val, "left")) ||
-                (!g_strcasecmp (str_val, "right")) ||
-                (!g_strcasecmp (str_val, "bottom")) ||
-                (!g_strcasecmp (str_val, "top")))
+            if ((!g_ascii_strcasecmp (str_val, "left")) ||
+                (!g_ascii_strcasecmp (str_val, "right")) ||
+                (!g_ascii_strcasecmp (str_val, "bottom")) ||
+                (!g_ascii_strcasecmp (str_val, "top")))
             {
                 if (settings->priv->navigationbar_position)
                     g_free (settings->priv->navigationbar_position);
                 settings->priv->navigationbar_position = g_strdup (str_val);
             }
             break;
-        case PROP_REVERT_ZOOM_DIRECTION:
-            settings->priv->revert_zoom_direction = g_value_get_boolean (value);
+        case PROP_INVERT_ZOOM_DIRECTION:
+            settings->priv->invert_zoom_direction = g_value_get_boolean (value);
             break;
         case PROP_WINDOW_WIDTH:
             settings->priv->window_width = g_value_get_uint (value);
@@ -779,14 +805,17 @@ rstto_settings_get_property    (GObject    *object,
         case PROP_SHOW_CLOCK:
             g_value_set_boolean (value, settings->priv->show_clock);
             break;
-        case PROP_HIDE_THUMBNAILBAR_FULLSCREEN:
-            g_value_set_boolean (value, settings->priv->hide_thumbnailbar_fullscreen);
+        case PROP_LIMIT_QUALITY:
+            g_value_set_boolean (value, settings->priv->limit_quality);
+            break;
+        case PROP_HIDE_THUMBNAILS_FULLSCREEN:
+            g_value_set_boolean (value, settings->priv->hide_thumbnails_fullscreen);
             break;
         case PROP_NAVBAR_POSITION:
             g_value_set_string (value, settings->priv->navigationbar_position);
             break;
-        case PROP_REVERT_ZOOM_DIRECTION:
-            g_value_set_boolean (value, settings->priv->revert_zoom_direction);
+        case PROP_INVERT_ZOOM_DIRECTION:
+            g_value_set_boolean (value, settings->priv->invert_zoom_direction);
             break;
         case PROP_WINDOW_WIDTH:
             g_value_set_uint (value, settings->priv->window_width);
